==> scanbot_sdk-2.8.2-beta3.tar.gz: ios/Classes/Classicial Component/CameraNativeView.swift <==
//
//  CameraNativeView.swift
//  Pods
//
//  Created by Sarbjyot Chahal on 31.03.22.
//

import Foundation
import Flutter
import ScanbotSDK
class CameraNativeView : NSObject, FlutterPlatformView, SBSDKBarcodeScannerViewControllerDelegate, CallErrorHandler {
    
    private let kViewControllerAppearDelay = 0.2
    private var cintainerView: UIView
    private var shouldDetectBarcodes: Bool = true
    private var waitingForImageSnap: Bool = false
    private var viewController: SBSDKBarcodeScannerViewController?
    private var cameraMethodChannel: FlutterMethodChannel?
    private let detectorChannel: FlutterMethodChannel?
    private let finderChannel: FlutterMethodChannel?
    private let configuration : [String: Any?]?
    private var imageGenerationType: SBSDKBarcodeImageGenerationType = SBSDKBarcodeImageGenerationType.none
    private var cachedCodes: [SBSDKBarcodeScannerResult]? = nil
    
    init(
        frame: CGRect,
        viewIdentifier viewId: Int64,
        arguments args: Any?,
        binaryMessenger messenger: FlutterBinaryMessenger
    ) {

        detectorChannel = FlutterMethodChannel(name: "scanbot_sdk_camera.detector_\(viewId)", binaryMessenger:  messenger)
        cameraMethodChannel = FlutterMethodChannel(name: "scanbot_sdk_camera_\(viewId)", binaryMessenger:  messenger)
        finderChannel = FlutterMethodChannel(name: MethodChannelConstants.kClassicalCameraFinderChannel, binaryMessenger:  messenger)

        cintainerView = UIView()
        configuration = args as? [String:Any?]
        super.init()
        
        DispatchQueue.main.asyncAfter(deadline: .now() + kViewControllerAppearDelay) { [weak self] in
            guard let _containerView = self?.cintainerView else {
                return
            }
            self?.createNativeView(view: _containerView)
        }
        detectorChannel?.setMethodCallHandler({
            (call: FlutterMethodCall, result: @escaping FlutterResult) -> Void in
            self.handleDetectorCall(call: call, result: result)
        })
        
        cameraMethodChannel?.setMethodCallHandler({
            (call: FlutterMethodCall, result: @escaping FlutterResult) -> Void in
            self.handleCameraCall(call: call, result: result)
        })
        
    }
    
    func view() -> UIView {
        return cintainerView
    }
    
    fileprivate func sendLicenseErrorToDetector(_ callHandler: FlutterMethodChannel) {
        let isValid = ScanbotSDK.isLicenseValid()
        let status = ScanbotSDK.licenseStatus()
        let expirationDate = ScanbotSDK.licenseExpirationDate()
        
        var dictionary: [String: Any?] = ["isLicenseValid": isValid, "status": status.stringName]
        if let date = expirationDate {
            let formatter = DateFormatter()
            formatter.timeZone = TimeZone(abbreviation: "UTC")
            formatter.dateFormat = "YYYY-MM-dd HH:mm:ss"
            formatter.locale = Locale(identifier: "en_EN")
            let dateString = formatter.string(from: date)
            dictionary["expirationDate"] = dateString
        }
        dictionary["message"] = "license is no more valid for this feature"
        callHandler.invokeMethod("onLicenseError", arguments: dictionary.encodedString)
    }
    
    func createNativeView(view _view: UIView){        
        DispatchQueue.main.async { [weak self] in

            if(!ScanbotSDK.isLicenseValid()){
                DispatchQueue.main.async { [weak self] in
                    guard let callHandler = self?.detectorChannel else {
                        fatalError("callHandler cannot be nil")
                    }
                    self?.sendLicenseErrorToDetector(callHandler)
                }
                return
            }
            
            guard let rootViewController = UIApplication.shared.keyWindow?.rootViewController else {
                fatalError("Cannot create native view: application keyWindow is NULL")
            }
            
            self?.viewController = SBSDKBarcodeScannerViewController(parentViewController: rootViewController,
                                                                     parentView: _view)
            self?.viewController?.delegate = self
            self?.setBarcodeScannerConfiguration(configuration:self?.configuration)
            self?.cameraMethodChannel?.invokeMethod(MethodChannelConstants.kOnCameraOpenMethod, arguments:"")
        }
        
    }
    
    func dispose(){
        viewController?.dismiss(animated: true)
        viewController = nil
    }
    
    func barcodeScannerController(_ controller: SBSDKBarcodeScannerViewController, didCaptureBarcodeImage barcodeImage: UIImage) {
        // to snap picture
        
        cameraMethodChannel?.invokeMethod("onShowHeavyProcessing", arguments: true)
        
        guard let uuid = SBSDKUIPageFileStorage.default().add(barcodeImage) else { return  }
        let uri = SBSDKUIPageFileStorage.default().imageURL(withPageFileID: uuid, pageFileType: .original)
        
        guard let codes = cachedCodes else { return }
        sendDetectedBarcodesResult(codes, uri?.standardizedFileURL.absoluteString ?? "")
        
        cameraMethodChannel?.invokeMethod("onShowHeavyProcessing", arguments: false)
        waitingForImageSnap = false
    }
    
    func barcodeScannerControllerShouldDetectBarcodes(_ controller: SBSDKBarcodeScannerViewController) -> Bool {
        return shouldDetectBarcodes && !waitingForImageSnap
    }
    
    func didChangeViewFinderRect(rect: CGRect) {
        let dictionary: [String: Any?] = ["left": rect.minX,
                                          "top": rect.minY,
                                          "right": rect.maxX,
                                          "bottom": rect.maxY,]
        
       finderChannel?.invokeMethod("finderSizeChanged", arguments: dictionary)
    }
    
    fileprivate func sendDetectedBarcodesResult(_ codes: [SBSDKBarcodeScannerResult],_ barcodeImageURI: String?) {
        let resultDict: [String: Any?] = ["barcodeItems": codes.map({ $0.encodedDictionary }), "barcodeImageURI" : barcodeImageURI]
        
        DispatchQueue.main.async { [weak self] in
            guard let callHandler = self?.detectorChannel else {
                fatalError("callHandler cannot be nil")
            }
            callHandler.invokeMethod("onBarcodeDetected", arguments: resultDict.encodedString)
        }
    }
    
    func barcodeScannerController(_ controller: SBSDKBarcodeScannerViewController, didDetectBarcodes codes: [SBSDKBarcodeScannerResult], on image: UIImage) {
        if(!codes.isEmpty){
            switch(imageGenerationType){
            case .none:
                sendDetectedBarcodesResult(codes, nil)
            case .fromVideoFrame:
                waitingForImageSnap = true
                cachedCodes = codes
            case .capturedImage:
                waitingForImageSnap = true
                cachedCodes = codes
            @unknown default:
                sendDetectedBarcodesResult(codes, nil)
            }
        }
    }
    
    
    func setBarcodeScannerConfiguration(configuration:[String:Any?]?) {
        
        viewController?.viewFinderRectDidChangeHandler = {(finderRect:CGRect) -> Void in
            self.didChangeViewFinderRect(rect:finderRect)
        }
        
        if let flashEnabled = configuration?["flashEnabled"] as? Bool {
            viewController?.isFlashLightEnabled = flashEnabled
        }
        
        if let cameraZoomFactor = configuration?["cameraZoomFactor"] as? CGFloat {
            let zoomConfig = SBSDKBaseScannerZoomConfiguration.default()
            zoomConfig.initialZoomFactor = cameraZoomFactor
            viewController?.zoomConfiguration = zoomConfig
        }
        
        if let finderConfiguration: [String:Any] = configuration?["finder"] as? [String : Any] {
            let finderConfig = SBSDKBaseScannerViewFinderConfiguration.default()
            finderConfig.backgroundColor = UIColor(hexString: "#00000000")
            finderConfig.lineColor = UIColor(hexString: "#00000000")
            finderConfig.isViewFinderEnabled = true
        
            
            if
                let finderAspectRatio = finderConfiguration["finderAspectRatio"] as? [String:Any],
                let width = finderAspectRatio["width"] as? Double,
                let height = finderAspectRatio["height"] as? Double
            {
                finderConfig.aspectRatio = SBSDKAspectRatio(width: width, andHeight: height)
            }
            
            if let padding = finderConfiguration["finderMinimumPadding"] as? CGFloat {
                finderConfig.minimumInset.top = padding
                finderConfig.minimumInset.bottom = padding
                finderConfig.minimumInset.left = padding
                finderConfig.minimumInset.right = padding
            }
            viewController?.viewFinderConfiguration = finderConfig
        }
        if let scannerConfiguration: [String:Any] = configuration?["scannerConfiguration"] as? [String : Any] {
            
            if scannerConfiguration["acceptedDocumentFormats"] is [String] {
                viewController?.acceptedDocumentTypes = SBSDKBarcodeDocumentType.fromArguments(scannerConfiguration)
            }

            if scannerConfiguration["barcodeFormats"] is [String] {
                viewController?.acceptedBarcodeTypes = SBSDKBarcodeType.fromArguments(scannerConfiguration)
            }
            
            if scannerConfiguration["engineMode"] is String {
                viewController?.engineMode = SBSDKBarcodeEngineMode.fromArguments(scannerConfiguration)!
            }
            
            if let additionalParams: [String : Any] = scannerConfiguration["additionalParameters"] as? [String : Any] {
                viewController?.additionalDetectionParameters = SBSDKBarcodeAdditionalParameters(from:additionalParams)
            }
            
            if let barcodeImageGenerationType = scannerConfiguration["barcodeImageGenerationType"] as? String {
                imageGenerationType = SBSDKBarcodeImageGenerationType(from:barcodeImageGenerationType)
                viewController?.barcodeImageGenerationType = imageGenerationType
            }
        }
        
    }
    
    fileprivate func toggleFlash(arguments : [String:Any?]?) {
        if let flashEnabled = arguments?["enabled"] as? Bool {
            viewController?.isFlashLightEnabled = flashEnabled
        }
    }
    
    fileprivate func isFlashAvailable(result: @escaping FlutterResult) {
        result(viewController?.isFlashLightAvailable)
    }
    
    func handleCameraCall(call: FlutterMethodCall, result: @escaping FlutterResult) {
        let arguments = call.arguments as? [String:Any?]
        switch call.method {
        case "flashEnabled" : toggleFlash(arguments: arguments)
        case "isFlashAvailable" : isFlashAvailable(result: result)
        case "dispose" : self.dispose()
        default:
            self.returnUnknownCallError(call: call, result: result)
            return
        }
        resurnSuccess(result: result)
    }
    
    func handleDetectorCall(call: FlutterMethodCall, result: @escaping FlutterResult) {
        switch call.method {
        case "enable" : shouldDetectBarcodes = true
        case "disable" : shouldDetectBarcodes = false
        default:
            self.returnUnknownCallError(call: call, result: result)
            return
        }
        resurnSuccess(result: result) //always return some result to the flutter
    }
    
    func resurnSuccess(result: @escaping FlutterResult){
        result("success")
    }
}

==> scanbot_sdk-2.11.0-beta3.tar.gz: ios/Classes/Classicial Component/CameraNativeView.swift <==
//
//  CameraNativeView.swift
//  Pods
//
//  Created by Sarbjyot Chahal on 31.03.22.
//

import Foundation
import Flutter
import ScanbotSDK
class CameraNativeView : NSObject, FlutterPlatformView, CallErrorHandler {
    
    private let kViewControllerAppearDelay = 0.2
    private var cintainerView: UIView
    private let configuration : [String: Any?]?
    private var cameraMethodChannel: FlutterMethodChannel?
    private let detectorChannel: FlutterMethodChannel?
    private let finderChannel: FlutterMethodChannel?

    private var scanner: SBSDKScanner?
    
    init(
        frame: CGRect,
        viewIdentifier viewId: Int64,
        arguments args: Any?,
        binaryMessenger messenger: FlutterBinaryMessenger
    ) {
        
        detectorChannel = FlutterMethodChannel(name: "scanbot_sdk_camera.detector_\(viewId)", binaryMessenger:  messenger)
        cameraMethodChannel = FlutterMethodChannel(name: "scanbot_sdk_camera_\(viewId)", binaryMessenger:  messenger)
        finderChannel = FlutterMethodChannel(name: MethodChannelConstants.kClassicalCameraFinderChannel, binaryMessenger:  messenger)
        
        cintainerView = UIView()
        configuration = args as? [String:Any?]
      
        switch configuration!["detectorType"] as? String {
        case "barcode" :
            scanner = BarcodeScanner(cameraMethodChannel: cameraMethodChannel, detectorChannel: detectorChannel, finderChannel: finderChannel, configuration: configuration!)
        case "document" :
            scanner = DocumentScanner(cameraMethodChannel: cameraMethodChannel, detectorChannel: detectorChannel, finderChannel: finderChannel, configuration: configuration!)
        case "mc" :
            scanner = MedicalCertificateScanner(cameraMethodChannel: cameraMethodChannel, detectorChannel: detectorChannel, finderChannel: finderChannel, configuration: configuration!)
        default: break
        }

        super.init()
        self.scanner?.attach(view: cintainerView)
    
        self.cameraMethodChannel?.setMethodCallHandler({
            (call: FlutterMethodCall, result: @escaping FlutterResult) -> Void in
            self.handleCameraCall(call: call, result: result)
        })
    
    }
    
    func view() -> UIView {
        return cintainerView
    }
    
    
    fileprivate func toggleFlash(arguments : [String:Any?]?) {
        if let flashEnabled = arguments?["enabled"] as? Bool {
            self.scanner?.setFlash(enabled: flashEnabled)
        }
    }
    
    fileprivate func isFlashAvailable(result: @escaping FlutterResult) {
        result(self.scanner?.isFlashAvailable())
    }
    
    func handleCameraCall(call: FlutterMethodCall, result: @escaping FlutterResult) {
        let arguments = call.arguments as? [String:Any?]
        switch call.method {
        case "flashEnabled" : toggleFlash(arguments: arguments)
        case "isFlashAvailable" : isFlashAvailable(result: result)
         return
        case "stopPreview" : self.scanner?.stopPreview()
        case "resumePreview" : self.scanner?.resumePreview()
        case "dispose" : self.scanner?.detach()
        default:
            self.returnUnknownCallError(call: call, result: result)
            return
        }
        resurnSuccess(result: result)
    }
    
    func resurnSuccess(result: @escaping FlutterResult){
        result("success")
    }
}
